package com.jacobs.client;

import com.jacobs.dataSource.DataConstants;
import com.jacobs.dataSource.DataFieldIDs;
import com.jacobs.pubsub.Publisher;
import com.jacobs.reservation.EventTypes;
import com.jacobs.reservation.ReservationQueryRequest;
import com.jacobs.reservation.ReservationTicketsAvailable;
import com.jacobs.reservation.TicketReservationRequest;
import java.awt.Color;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Observable;
import java.util.Observer;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;

/**
 * This ReservationControlPanel contains the controls that allows the user
 * to interface with the reservation system.  Users can query for music event
 * information and can make reservation requests.
 *
 * @author Kurt Jacobs
 *
 * Permission is hereby granted to use, copy, modify and/or distribute
 * this software for commercial and non-commercial purposes.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY. ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE TO ANY PARTY FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE.
 *
 * @see com.jacobs.pubsub.Publisher
 * @see com.jacobs.pubsub.Event
 * @see java.util.Observer
 * @see java.util.Observable
 */


public class ReservationControlPanel extends JPanel implements DataFieldIDs, EventTypes
{

    private JComboBox mMusicGenreComboBox = null;
    private JButton mFindMusicEventsButton = null;
    private JButton mReserveTicketsButton = null;
    private JTextField mTicketsRequestedTextField = null;
    private Publisher mPublisher = null;
    private TicketsAvailableSubscriber mTicketsAvailableSubscriber = null;
    private FindGenreActionListener mFindGenreActionListener = null;
    private ReserveTicketsActionListener mReserveTicketsActionListener = null;
    private int mTicketsAvailable = 0;
    private static final Font DEFAULT_TITLE_FONT = new Font("Dialog", 1, 12);
    private static final Font DEFAULT_DIALOG_FONT = new Font("Dialog", 0, 12);

    private final class FindGenreActionListener implements ActionListener
    {
        public void actionPerformed(ActionEvent actionEvent)
        {
            String queryString;
            ReservationQueryRequest queryRequest = new ReservationQueryRequest();
            queryString = FIELD_IDS[GENRE_ID] + "=" + mMusicGenreComboBox.getSelectedItem();
            queryRequest.setQuery(queryString);
            mPublisher.publish(queryRequest);
        }
    }


    private final class ReserveTicketsActionListener implements ActionListener
    {
        public void actionPerformed(ActionEvent actionEvent)
        {
            int ticketsRequested = 0;
            try
            {
                ticketsRequested = Integer.parseInt(mTicketsRequestedTextField.getText());
            }
            catch (NumberFormatException numberFormatException)
            {
            }
            // If the value fetched was 0 or if the
            // number did not parse correct (and the
            // value is it's initialzed value of 0)
            // then post an error message.
            if (ticketsRequested == 0)
            {
                postWarningMessage("Ticket Request Error",
                    "Please enter a value greater than zero and\n less than/equal to the number of tickets available");
            }
            else if (ticketsRequested > mTicketsAvailable)
            {
                postWarningMessage("Ticket Request Error",
                    "Tickets requested is greater than the\nnumber of tickets available");
            }
            else
            {
                mPublisher.publish(
                    new TicketReservationRequest(ticketsRequested));
            }
        }
    }


    private void postWarningMessage(String title, String message)
    {
        JOptionPane.showMessageDialog(null, message, title, JOptionPane.WARNING_MESSAGE);
    }

    private final class TicketsAvailableSubscriber implements Observer
    {
        private ReservationTicketsAvailable mReservationTicketsAvailable = null;

        public TicketsAvailableSubscriber()
        {
            mPublisher.subscribe(this, TICKETS_AVAILABLE_EVENT);
        }

        public void update(Observable observable, Object reservationEvent)
        {
            mReservationTicketsAvailable = (ReservationTicketsAvailable)reservationEvent;
            mTicketsAvailable = mReservationTicketsAvailable.getAvailableTickets();
            if (mTicketsAvailable > 0)
            {
                setReservationControlsEnable(true);
            }
            else
            {
                setReservationControlsEnable(false);
            }
        }
    }


    private void setReservationControlsEnable(boolean requestedState)
    {
        mTicketsRequestedTextField.setEnabled(requestedState);
        mReserveTicketsButton.setEnabled(requestedState);
    }

    public ReservationControlPanel()
    {
        mMusicGenreComboBox = new JComboBox();
        mFindMusicEventsButton = new JButton();
        mReserveTicketsButton = new JButton();
        mTicketsRequestedTextField = new JTextField(5);
        mFindGenreActionListener = new FindGenreActionListener();
        mReserveTicketsActionListener = new ReserveTicketsActionListener();
        initComponents();
        populateMusicGenreChoices();
        addListeners();
        mPublisher = Publisher.getInstance();
        mTicketsAvailableSubscriber = new TicketsAvailableSubscriber();
    }

    private void populateMusicGenreChoices()
    {
        int musicGenres = 0;
        mMusicGenreComboBox.removeAllItems();
        mMusicGenreComboBox.addItem(DataConstants.WILDCARD);
        musicGenres = DataConstants.FIELD_CATEGORIES.length;
        for (int i = 0; i < musicGenres; i++)
        {
            mMusicGenreComboBox.addItem(DataConstants.FIELD_CATEGORIES[i]);
        }
    }

    private void initComponents()
    {
        add(createFindMusicEventsPanel(),
            new GridBagConstraints(0, 0, 1, 1, 0.5D, 0.0D, 10, 0,
            new Insets(0, 5, 0, 5), 0, 0));
        add(createReserveTicketsPanel(),
            new GridBagConstraints(1, 0, 1, 1, 0.5D, 0.0D, 10, 1,
            new Insets(0, 0, 0, 5), 0, 0));
    }

    private JPanel createFindMusicEventsPanel()
    {
        JPanel findMusicEventTitledPanel = new JPanel();
        JLabel musicGenreLabel = new JLabel();
        TitledBorder findMusicEventTitledBorder = new TitledBorder("Find Music Event");
        findMusicEventTitledBorder.setTitleFont(DEFAULT_TITLE_FONT);
        findMusicEventTitledBorder.setTitleColor(Color.black);
        setLayout(
            new GridBagLayout());
        findMusicEventTitledPanel.setBorder(findMusicEventTitledBorder);
        findMusicEventTitledPanel.setLayout(
            new GridBagLayout());
        musicGenreLabel.setText("Music Genre");
        musicGenreLabel.setForeground(Color.black);
        musicGenreLabel.setFont(DEFAULT_TITLE_FONT);
        mMusicGenreComboBox.setFont(DEFAULT_DIALOG_FONT);
        findMusicEventTitledPanel.add(musicGenreLabel,
            new GridBagConstraints(0, 0, 1, 1, 0.0D, 0.0D, 10, 0,
            new Insets(0, 5, 0, 5), 0, 0));
        findMusicEventTitledPanel.add(mMusicGenreComboBox,
            new GridBagConstraints(0, 1, 1, 1, 0.0D, 0.0D, 10, 2,
            new Insets(0, 5, 0, 5), 0, 0));
        mFindMusicEventsButton.setFont(DEFAULT_DIALOG_FONT);
        mFindMusicEventsButton.setText("Find Event(s)");
        findMusicEventTitledPanel.add(mFindMusicEventsButton,
            new GridBagConstraints(0, 2, 1, 1, 0.0D, 0.0D, 10, 2,
            new Insets(5, 5, 5, 5), 0, 0));
        mFindMusicEventsButton.setRequestFocusEnabled(false);
        mMusicGenreComboBox.setRequestFocusEnabled(false);
        return findMusicEventTitledPanel;
    }

    private JPanel createReserveTicketsPanel()
    {
        JPanel reserveTicketsTitledPanel = new JPanel();
        JLabel numberTicketsReservedLabel = new JLabel();
        TitledBorder reserveTicketsTitledBorder = new TitledBorder("Ticket Reservation");
        reserveTicketsTitledBorder.setTitleFont(DEFAULT_TITLE_FONT);
        reserveTicketsTitledBorder.setTitleColor(Color.black);
        reserveTicketsTitledPanel.setBorder(reserveTicketsTitledBorder);
        reserveTicketsTitledPanel.setLayout(
            new GridBagLayout());
        JPanel ticketReservePanel = new JPanel();
        ticketReservePanel.setLayout(
            new GridBagLayout());
        numberTicketsReservedLabel.setText("Number of Tickets :");
        numberTicketsReservedLabel.setFont(DEFAULT_DIALOG_FONT);
        numberTicketsReservedLabel.setForeground(Color.black);
        mTicketsRequestedTextField.setText("1");
        mReserveTicketsButton.setFont(DEFAULT_DIALOG_FONT);
        mReserveTicketsButton.setText("Reserve Tickets(s)");
        ticketReservePanel.add(mTicketsRequestedTextField,
            new GridBagConstraints(1, 1, 1, 2, 1.0D, 0.0D, 10, 0,
            new Insets(0, 0, 5, 0), 25, 0));
        ticketReservePanel.add(numberTicketsReservedLabel,
            new GridBagConstraints(0, 1, 1, 2, 1.0D, 0.0D, 18, 0,
            new Insets(2, 3, 5, 3), 0, 0));
        ticketReservePanel.add(mReserveTicketsButton,
            new GridBagConstraints(0, 3, 2, 1, 1.0D, 0.0D, 18, 1,
            new Insets(0, 0, 0, 0), 0, 0));
        reserveTicketsTitledPanel.add(ticketReservePanel,
            new GridBagConstraints(0, 0, 1, 1, 0.0D, 0.0D, 10, 0,
            new Insets(15, 5, 0, 5), 0, 0));
        mReserveTicketsButton.setRequestFocusEnabled(false);
        setReservationControlsEnable(false);
        return reserveTicketsTitledPanel;
    }

    private void addListeners()
    {
        mFindMusicEventsButton.addActionListener(mFindGenreActionListener);
        mReserveTicketsButton.addActionListener(mReserveTicketsActionListener);
    }
}

