package com.jacobs.client;

import com.jacobs.pubsub.Publisher;
import com.jacobs.reservation.EventTypes;
import com.jacobs.reservation.ReservationConfirmation;
import com.jacobs.reservation.ReservationManager;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Observable;
import java.util.Observer;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.UIManager;

/**
 * ReservationMain contains all the child GUI components needed to make
 * queries and reserve tickets.   ReservationMain also owns a
 * ReservationManager.
 * Note: The ReservationManager processes queries and publishes the
 * results of a reservation.
 *
 * @author Kurt Jacobs
 *
 * Permission is hereby granted to use, copy, modify and/or distribute
 * this software for commercial and non-commercial purposes.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY. ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE TO ANY PARTY FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE.
 *
 * @see com.jacobs.pubsub.Publisher
 * @see com.jacobs.pubsub.Event
 * @see java.util.Observer
 * @see java.util.Observable
 */
public class ReservationMain extends JFrame implements EventTypes
{
    private ReservationControlPanel mReservationControlPanel = null;
    private ReservationQueryResultsPanel mReservationQueryResultsPanel = null;
    private ReservationStatusPanel mReservationStatusPanel = null;
    private ReservationWindowAdapter mReservationWindowAdapter = null;
    private Publisher mPublisher = null;
    private ReservationConfirmationSubscriber mReservationConfirmationSubscriber = null;
    private static final int INITIAL_FORM_WIDTH = 550;
    private static final int INITIAL_FORM_HEIGHT = 400;


    private final class ReservationWindowAdapter extends WindowAdapter
    {
        public void windowClosing(WindowEvent windowEvent)
        {
            exitForm();
        }
    }


    public ReservationMain()
    {
        mReservationControlPanel = new ReservationControlPanel();
        mReservationQueryResultsPanel = new ReservationQueryResultsPanel();
        mReservationStatusPanel = new ReservationStatusPanel();
        mReservationWindowAdapter = new ReservationWindowAdapter();
        initComponents();
        addListeners();
        mPublisher = Publisher.getInstance();
        mReservationConfirmationSubscriber = new ReservationConfirmationSubscriber();

        pack();
        setSize(INITIAL_FORM_WIDTH, INITIAL_FORM_HEIGHT);
        show();

    }

    private void initComponents()
    {
        getContentPane().setLayout(
            new GridBagLayout());
        getContentPane().add(mReservationControlPanel,
            new GridBagConstraints(0, 0, 1, 1, 0.0D, 0.0D, 18, 0,
            new Insets(5, 5, 5, 5), 0, 0));
        getContentPane().add(mReservationQueryResultsPanel,
            new GridBagConstraints(0, 1, 1, 1, 1.0D, 1.0D, 18, 1,
            new Insets(5, 5, 5, 5), 0, 0));
        getContentPane().add(mReservationStatusPanel,
            new GridBagConstraints(0, 2, 1, 1, 0.5D, 0.0D, 18, 0,
            new Insets(5, 5, 5, 5), 0, 0));
    }

    private void addListeners()
    {
        addWindowListener(mReservationWindowAdapter);
    }

    private void exitForm()
    {
        dispose();
        System.exit(0);
    }

    private final class ReservationConfirmationSubscriber implements Observer
    {
        private JOptionPane mConfirmationDialog = null;
        private ReservationConfirmation mReservationConfirmation = null;

        public ReservationConfirmationSubscriber()
        {
            mPublisher.subscribe(this, RESERVATION_CONFIRMATION_EVENT);
        }

        public void update(Observable observable, Object reservationEvent)
        {
            mReservationConfirmation = (ReservationConfirmation)reservationEvent;
            JOptionPane.showMessageDialog(null, mReservationConfirmation.getConfirmationSummary(),
                "Reservation Confirmation", JOptionPane.INFORMATION_MESSAGE);
        }
    }


    public static void main(String[] args)
    {
        try
        {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        }
        catch (Exception exception)
        {
            System.err.println("Could not set look/feel for platform.");
        }
        ReservationMain reservationMain = new ReservationMain();
        ReservationManager reservationManager = new ReservationManager();
    }
}


