package com.jacobs.client;

import com.jacobs.dataSource.DataFieldIDs;
import com.jacobs.dataSource.DatabaseRecord;
import com.jacobs.pubsub.Publisher;
import com.jacobs.reservation.EventTypes;
import com.jacobs.reservation.ReservationDataUpdate;
import com.jacobs.reservation.ReservationQueryResponse;
import com.jacobs.reservation.ReservationTicketsAvailable;
import com.jacobs.reservation.TicketsAvailableStatus;
import java.util.Collection;
import java.util.Iterator;
import java.util.Observable;
import java.util.Observer;
import java.util.Vector;
import javax.swing.table.AbstractTableModel;

/**
 * The ReservationQueryModel holds the data pertaining to upcoming events.
 *
 *
 * @author Kurt Jacobs
 *
 * Permission is hereby granted to use, copy, modify and/or distribute
 * this software for commercial and non-commercial purposes.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY. ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE TO ANY PARTY FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE.
 *
 * @see com.jacobs.pubsub.Publisher
 * @see com.jacobs.pubsub.Event
 * @see java.util.Observer
 * @see java.util.Observable
 */

public class ReservationQueryResultsModel extends AbstractTableModel implements DataFieldIDs, EventTypes
{

    private String[] mColumnHeaders = null;
    private Publisher mPublisher = null;
    private QueryResponseSubscriber mQueryResponseSubscriber = null;
    private ReservationDataUpdateSubscriber mReservationDataUpdateSubscriber = null;
    private Vector mReservationData = null;

    public ReservationQueryResultsModel()
    {
        mReservationData = new Vector();
        mPublisher = Publisher.getInstance();
        mQueryResponseSubscriber = new QueryResponseSubscriber();
        mReservationDataUpdateSubscriber = new ReservationDataUpdateSubscriber();
    }

   void updateReservationData(Collection data)
    {
        Iterator iterator = null;
        mReservationData.removeAllElements();
        iterator = data.iterator();
        while (iterator.hasNext())
        {
            mReservationData.addElement((DatabaseRecord)iterator.next());
        }
        fireTableDataChanged();
    }

   void updateReservationData(DatabaseRecord data)
    {
        Iterator iterator = null;
        boolean recordNotFound = true;
        int dataIndex = 0;
        DatabaseRecord record = null;
        String searchKey = null;
        if (mReservationData != null)
        {
            iterator = mReservationData.iterator();
            searchKey = data.getEventID();
            while ((iterator.hasNext() && recordNotFound))
            {
                record = (DatabaseRecord)iterator.next();
                if (searchKey.equals(record.getEventID()))
                {
                    record.setTickets(data.getTickets());
                    recordNotFound = false;
                }
                else
                {
                    ++dataIndex;
                }
            }
            if (!recordNotFound)
            {
                fireTableRowsUpdated(dataIndex, dataIndex);
                mPublisher.publish(
                    new ReservationTicketsAvailable(searchKey, record.getTickets()));
                mPublisher.publish(
                    new TicketsAvailableStatus(data.getTickets()));
            }
        }
    }

    private final class QueryResponseSubscriber implements Observer
    {
        private Vector queryResult = null;
        private Collection queryResults;

        public QueryResponseSubscriber()
        {
            mPublisher.subscribe(this, QUERY_RESPONSE_EVENT);
        }

        public void update(Observable observable, Object reservationEvent)
        {
            queryResults = ((ReservationQueryResponse)reservationEvent).getQueryResult();
            updateReservationData(queryResults);
        }
    }


    private final class ReservationDataUpdateSubscriber implements Observer
    {
        public ReservationDataUpdateSubscriber()
        {
            mPublisher.subscribe(this, DATA_UPDATE_EVENT);
        }

        public void update(Observable observable, Object reservationEvent)
        {
            updateReservationData(((ReservationDataUpdate)reservationEvent).getRecord());
        }
    }


    void setHeader(String[] tableHeader)
    {
        mColumnHeaders = tableHeader;
    }

    public int getRowCount()
    {
        return mReservationData.size();
    }

    public int getColumnCount()
    {
        return mColumnHeaders.length;
    }

    public String getColumnName(int columnIndex)
    {
        return mColumnHeaders[columnIndex];
    }

    public Class getColumnClass(int columnIndex)
    {
        return getValueAt(0, columnIndex).getClass();
    }

    public Object getValueAt(int rowIndex, int columnIndex)
    {
        Object databaseField = null;
        DatabaseRecord record = (DatabaseRecord)mReservationData.elementAt(rowIndex);
        switch (columnIndex)
        {
            case EVENT_ID:
                databaseField = record.getEventID();
                break;
            case PERFORMER_ID:
                databaseField = record.getPerformer();
                break;
            case GENRE_ID:
                databaseField = record.getGenre();
                break;
            case DATE_ID:
                databaseField = record.getDate();
                break;
            case TIME_ID:
                databaseField = record.getTime();
                break;
            case TICKET_ID:
                databaseField = new Integer(record.getTickets());
                break;
            case PRICE_ID:
                databaseField = record.getPrice();
                break;
        }
        return databaseField;
    }
}
