package com.jacobs.client;

import com.jacobs.dataSource.DataFieldIDs;
import com.jacobs.pubsub.Publisher;
import com.jacobs.reservation.ReservationStatus;
import com.jacobs.reservation.ReservationTicketsAvailable;
import com.jacobs.reservation.TicketsAvailableStatus;
import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableColumn;

/**
 * The ReservationQueryResultsPanel displays the result of user queries in a table.
 *
 * @author Kurt Jacobs
 * Permission is hereby granted to use, copy, modify and/or distribute
 * this software for commercial and non-commercial purposes.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY. ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE TO ANY PARTY FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE.
 *
 * @see com.jacobs.pubsub.Publisher
 * @see com.jacobs.pubsub.Event
 * @see java.util.Observer
 * @see java.util.Observable
 */
public class ReservationQueryResultsPanel extends JPanel implements DataFieldIDs
{
    private Publisher mPublisher;
    private JScrollPane mReservationQueryDisplayScrollPane;
    private JTable mReservationQueryResultsTable;
    private ReservationQueryResultsModel mReservationQueryResultsModel;
    private ReservationTableModelListener mTableModelListener;
    private TableRowSelectedListener mMouseAdapter = new TableRowSelectedListener();
    private static final Font DEFAULT_DIALOG_FONT = new Font("Dialog", 0, 12);

    private class ReservationTableModelListener implements TableModelListener
    {
        public void tableChanged(TableModelEvent tableEvent)
        {
            if (tableEvent.getFirstRow() == -1)
            {
                sizeHeader();
            }
            // No row is selected, clear status
            else
            {
                mPublisher.publish(
                    new ReservationStatus(" "));
                mPublisher.publish(
                    new ReservationTicketsAvailable(null, 0));
            }
        }
    }


    public ReservationQueryResultsPanel()
    {
        mReservationQueryDisplayScrollPane = new JScrollPane();
        mReservationQueryResultsTable = new JTable();
        mReservationQueryResultsModel = new ReservationQueryResultsModel();
        mTableModelListener = new ReservationTableModelListener();
        initComponents();
        mReservationQueryResultsTable.setModel(mReservationQueryResultsModel);
        sizeHeader();
        addListeners();
        mPublisher = Publisher.getInstance();
    }

    private void initComponents()
    {
        setFont(DEFAULT_DIALOG_FONT);
        mReservationQueryDisplayScrollPane.getViewport().add(mReservationQueryResultsTable);
        setLayout(
            new BorderLayout());
        add(mReservationQueryDisplayScrollPane, "Center");
        mReservationQueryResultsModel.setHeader(FIELD_IDS);
        mReservationQueryResultsTable.setSelectionMode(0);
    }

    /** This method sets the width of the columns to accommodate the text of
    * the column heading.
    */
    private void sizeHeader()
    {
        FontMetrics mFontMetrics = mReservationQueryResultsTable.getFontMetrics(mReservationQueryResultsTable.getFont());
        TableColumn column = null;
        String columnName = null;
        int columnNumber = 0;
        columnNumber = mReservationQueryResultsModel.getColumnCount();
        int width = 0;
        for (int headerIndex = 0; headerIndex < columnNumber; headerIndex++)
        {
            column = mReservationQueryResultsTable.getColumnModel().getColumn(headerIndex);
            columnName = mReservationQueryResultsModel.getColumnName(headerIndex);
            width = mFontMetrics.stringWidth(columnName);
            column.setPreferredWidth(width);
            column.setWidth(width);
        }
    }

    // Add listeners to the GUI components
    private void addListeners()
    {
        mReservationQueryResultsModel.addTableModelListener(mTableModelListener);
        mReservationQueryResultsTable.addMouseListener(mMouseAdapter);
    }

    /**
     * This listener listens for row selections made within the table that is displayed on this ReservationQueryResultsPanel.
     * When a row is selected, events are dispatch via the EventDispatcher
     * to let external object know about the contents the current selection
     */
    private final class TableRowSelectedListener extends MouseAdapter
    {
        public void mouseReleased(MouseEvent e)
        {
            int ticketsAvailableColumn = 0;
            int keyColumn = 0;
            int selectedRow = 0;
            int ticketsAvailable = 0;
            String statusString = null;
            String recordKey = null;
            Integer ticketsObject = null;
            // If the table is currently disabled then ignore mouse events
            if (mReservationQueryResultsTable.isEnabled())
            {
                ticketsAvailableColumn = mReservationQueryResultsModel.findColumn(FIELD_IDS[TICKET_ID]);
                keyColumn = mReservationQueryResultsModel.findColumn(FIELD_IDS[EVENT_ID]);
                if (ticketsAvailableColumn != -1)
                {
                    selectedRow = mReservationQueryResultsTable.getSelectedRow();
                    ticketsObject = (Integer)mReservationQueryResultsTable.getValueAt(selectedRow,
                        mReservationQueryResultsTable.convertColumnIndexToView(ticketsAvailableColumn));
                    ticketsAvailable = ticketsObject.intValue();
                    recordKey = (String)mReservationQueryResultsTable.getValueAt(selectedRow,
                        mReservationQueryResultsTable.convertColumnIndexToView(keyColumn));
                    mPublisher.publish(
                        new TicketsAvailableStatus(ticketsAvailable));
                    mPublisher.publish(
                        new ReservationTicketsAvailable(recordKey, ticketsAvailable));
                }
            }
        }
    }
}
