package com.jacobs.dataSource;

import java.text.NumberFormat;
import java.util.Locale;

/**
 * This class is used for the encapsulate currency.  It extends the
 * Number class so that external classes (like renderers) can deal with
 * it in an appropriate way (justifying the value to the right for example).
 * @author Kurt Jacobs
 *
 * Permission is hereby granted to use, copy, modify and/or distribute
 * this software for commercial and non-commercial purposes.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY. ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE TO ANY PARTY FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE.
 *
 */
public class Currency extends Number
{
    private Locale mLocation;
    private double mValue;

    /**
     * This constructor takes a double that represents a monetary value. The current implementation defautls to the US locale.
     * @param value a double value that represents an amount
     */
    public Currency(double value)
    {
        mLocation = Locale.US;
        setValue(value);
    }

    /** This method allows this class to have it's value changed after creation. */
    public void setValue(double value)
    {
        NumberFormat form = NumberFormat.getCurrencyInstance(mLocation);
        String formattedString = form.format(value);
        if (formattedString.equals("($0.00)"))
        {
            mValue = 0.0D;
        }
        else
        {
            mValue = value;
        }
    }

    /** This method allows the class user to change the locale attribute.  The default locale attribute is US. */
    void setLocale(Locale location)
    {
        mLocation = location;
    }

    /** This method formats the object according to the current locale. */
    public String toString()
    {
        NumberFormat form = NumberFormat.getCurrencyInstance(mLocation);
        return form.format(mValue);
    }

    // Implementation of abstract methods of Number

    /**
     * Returns the value of the specified number as an <code>int</code>. This may involve rounding.
     * @return  the numeric value represented by this object after conversion to type <code>int</code>.
     */
    public int intValue()
    {
        return (int)mValue;
    }

    /**
     * Returns the value of the specified number as a <code>long</code>. This may involve rounding.
     * @return  the numeric value represented by this object after conversion to type <code>long</code>.
     */
    public long longValue()
    {
        return (long)mValue;
    }

    /**
     * Returns the value of the specified number as a <code>float</code>. This may involve rounding.
     * @return  the numeric value represented by this object after conversion to type <code>float</code>.
     */
    public float floatValue()
    {
        return (float)mValue;
    }

    /**
     * Returns the value of the specified number as a <code>double</code>. This may involve rounding.
     * @return  the numeric value represented by this object after conversion to type <code>double</code>.
     */
    public double doubleValue()
    {
        return mValue;
    }

}
