package com.jacobs.reservation;

import com.jacobs.dataSource.Database;
import com.jacobs.dataSource.DatabaseRecord;
import com.jacobs.pubsub.Publisher;
import java.util.ArrayList;
import java.util.Observable;
import java.util.Observer;

/**
 * The ReservationManager subscribes to query requests and reservation requests.
 * After receiving a query request, the ReservationManager queries the database
 * and publishes the results of the query.  After receiving a  request for tickets,
 * the ReservationManager modifies the database and publishes a
 * ReservationConfirmation event.
 *
 * Note: This simplistic example is designed to illustrate an implmentation of
 * a Publisher Subscriber Pattern and does not address the complexities of
 * a genuine reservation system.
 *
 * @author Kurt Jacobs
 *
 * Permission is hereby granted to use, copy, modify and/or distribute
 * this software for commercial and non-commercial purposes. THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY. ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE TO ANY PARTY FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE.
 */

public class ReservationManager implements EventTypes
{
    private Publisher mPublisher = null;
    private QuerySubscriber mQuerySubscriber = null;
    private TicketsAvailableSubscriber mTicketsAvailableSubscriber = null;
    private TicketReservationRequestSubscriber mTicketReservationRequestSubscriber = null;
    private Database mDatabase = null;
    private String mRecordKey = null;
    private int mTicketsAvailable = 0;

    public ReservationManager()
    {
        mDatabase = new Database();
        mPublisher = Publisher.getInstance();
        mQuerySubscriber = new QuerySubscriber();
        mTicketsAvailableSubscriber = new TicketsAvailableSubscriber();
        mTicketReservationRequestSubscriber = new TicketReservationRequestSubscriber();
    }

    private final class TicketsAvailableSubscriber implements Observer
    {
        private ReservationTicketsAvailable mReservationTicketsAvailable;

        public TicketsAvailableSubscriber()
        {
            mPublisher.subscribe(this, TICKETS_AVAILABLE_EVENT);
        }

        public void update(Observable observable, Object reservationEvent)
        {
            mReservationTicketsAvailable = (ReservationTicketsAvailable)reservationEvent;
            mRecordKey = mReservationTicketsAvailable.getRecordKey();
            mTicketsAvailable = mReservationTicketsAvailable.getAvailableTickets();
        }
    }


    private final class TicketReservationRequestSubscriber implements Observer
    {
        private TicketReservationRequest mTicketReservationRequest = null;
        private DatabaseRecord mDatabaseRecord = null;

        public TicketReservationRequestSubscriber()
        {
            mPublisher.subscribe(this, TICKET_REQUEST_EVENT);
        }

        public void update(Observable observable, Object reservationEvent)
        {
            ReservationConfirmation reservationConfirmation = null;
            int ticketsRequested = 0;
            if (mRecordKey == null)
            {
                System.err.println("No record selected, request ignored");
            }
            else
            {
                mTicketReservationRequest = (TicketReservationRequest)reservationEvent;
                ticketsRequested = mTicketReservationRequest.getTicketsRequested();
                mDatabaseRecord = mDatabase.modifyRecord(mRecordKey, ticketsRequested);
                if (mDatabaseRecord != null)
                {
                    reservationConfirmation = new ReservationConfirmation();
                    reservationConfirmation.setConfirmationID(System.currentTimeMillis());
                    reservationConfirmation.setTicketsPurchased(ticketsRequested);
                    reservationConfirmation.setPricePerTicket(mDatabaseRecord.getPrice());
                    reservationConfirmation.setDate(mDatabaseRecord.getDate());
                    reservationConfirmation.setTime(mDatabaseRecord.getTime());
                    reservationConfirmation.setEventDescription(mDatabaseRecord.getPerformer());
                    mPublisher.publish(
                        new ReservationDataUpdate(mDatabaseRecord));
                    mPublisher.publish(reservationConfirmation);
                }
            }
        }
    }


    private final class QuerySubscriber implements Observer
    {
        private ArrayList mQueryResult = null;
        private String mQueryString = null;

        public QuerySubscriber()
        {
            mPublisher.subscribe(this, QUERY_REQUEST_EVENT);

        }

        public void update(Observable observable, Object reservationEvent)
        {
            mQueryString = ((ReservationQueryRequest)reservationEvent).getQuery();
            mQueryResult = mDatabase.findRecords(mQueryString);
            mPublisher.publish(
                new ReservationQueryResponse(mQueryResult));
        }
    }
}
